param (
    [string]$VCPath = "C:/Program Files (x86)/Microsoft Visual Studio 14.0/VC",
    [Parameter(Mandatory=$false)][ValidateSet("true", "false")][string]$UseSubwindows="false"
)

Function Parse-Bool([string]$BoolString){
	try{
        $ParsedBool = $false
        switch($BoolString.ToLower()) {
            "true" { $ParsedBool = $true }
            default { $ParsedBool = $false }
        }

        return [bool]$ParsedBool
    }
	catch{}
}

Function Try-CreateDirectory([string]$DirPath){
	try{
        if(-Not (Test-Path $DirPath))
        {
            New-Item -ItemType directory -Path $DirPath
        }
    }
	catch{}
}

# Invokes a Cmd.exe shell script and updates the environment.
function Invoke-CmdScript {
  param(
    [String] $EnvPath,
    [String] $ScriptName,
    [String] $WorkingDirectory,
    [bool] $UseSubwindows
  )

  Write-Host "Getting MSVC environment from $EnvPath"

  if($UseSubwindows)
  {
    $p = Start-Process "cmd" -ArgumentList "/c","`"$EnvPath`" & $ScriptName" -WorkingDirectory "$WorkingDirectory" -PassThru
    $p.WaitForExit()
  }
  else
  {
    $p = Start-Process "cmd" -ArgumentList "/c","`"$EnvPath`" & $ScriptName" -WorkingDirectory "$WorkingDirectory" -NoNewWindow -PassThru
    $p.WaitForExit()
  }
}

$ScriptDir = (Get-Item "$($PSScriptRoot)").FullName
$VCBinPath = Join-Path -Path $VCPath -ChildPath "bin"
$VCEnvBatPath = Join-Path -Path $VCPath -ChildPath "bin/amd64/vcvars64.bat"

# Parse bool to avoid conflicts with options
$UseSubwindowsBool = Parse-Bool $UseSubwindows

Write-Host "Building CarlaServer"

$CarlaServerBuildPath = (Join-Path -Path $ScriptDir -ChildPath Util/cmake/build)
Try-CreateDirectory $CarlaServerBuildPath

Invoke-CmdScript $VCEnvBatPath "cmake ^ -DCMAKE_GENERATOR_PLATFORM=x64 ^ -DCMAKE_BUILD_TYPE=Release ^ -DCMAKE_INSTALL_PREFIX=../../../Unreal/CarlaUE4/Plugins/Carla/CarlaServer/ ^ .. & cmake --build . --target INSTALL --config Release & exit" $CarlaServerBuildPath $UseSubwindowsBool

Write-Host "Building CarlaServer - Done"